# Author: Yuji Sasaki <yuji@sasaki.dev>
# Copyright (c) 2022 Yuji Sasaki
# License: MIT License

from enum import Enum

class Category(Enum):
  NONE = 0
  TEXT = 1
  IMAGE = 2
  MOVIE = 3
  STAMP = 4
  FILE = 5
  CALL = 6
  CALL_CANCELLED = 7
  CALL_MISSED = 8
  CONTACT = 9
  UNSENT = 10
  POLL = 11

  @classmethod
  def fromLabel(cls, label: str):
    if label == '[写真]' or label == '[Photo]':
      return cls.IMAGE
    elif label == '[動画]' or label == '[Video]':
      return cls.MOVIE
    elif label == '[ファイル]' or label == '[File]':
      return cls.FILE
    elif label == '[スタンプ]' or label == '[Sticker]':
      return cls.STAMP
    elif label == '[連絡先]' or label == '[Contact]':
      return cls.CONTACT
    elif label == '[投票]' or label == '[Poll]':
      return cls.POLL
    else:
      return cls.NONE
    



# Author: Yuji Sasaki <yuji@sasaki.dev>
# Copyright (c) 2022 Yuji Sasaki
# License: MIT License

from datetime import datetime
from typing import List

from .category import Category

class Message:

  time: datetime
  username: str
  textlines: List[str]
  kind: Category

  def __init__(self, _datetime: datetime, _username: str, _text: list, _kind: Category = Category.TEXT):
    self.time = _datetime
    self.username = _username
    self.textlines = [_text]
    self.kind = _kind

  def addMessage(self, text: str) -> None:
    self.textlines.append(text)

  def asList(self) -> List[str]:
    return [self.time, self.username, self.kind, " ".join(self.textlines)]

  def __str__(self):
    return ",".join(map(lambda x: str(x), self.asList()))


# Author: Yuji Sasaki <yuji@sasaki.dev>
# Copyright (c) 2022 Yuji Sasaki
# License: MIT License

from datetime import datetime
from itertools import groupby
import re
from typing import List

from .message import Message
from .category import Category

class Reader:

  @classmethod
  def readFile(cls, file: str) -> list[Message]:
    lines = []
    with open(file) as f:
      lines = f.readlines()[2:]

    dateString = "2020/01/01"
    isNextNewDate = False
    messages: list[Message] = []

    for line in lines:
      if line == '\n':
        isNextNewDate = True
        continue
      
      if isNextNewDate and re.match('\d{4}/\d{2}/\d{2}', line):
        dateString = line[:10].replace('\n', '')
        isNextNewDate = False
        continue

      splitted = line.split('\t')

      if len(splitted) >= 3 and re.fullmatch(r'\d{2}:\d{2}', splitted[0]):
        time = datetime.strptime(f'{dateString} {splitted[0]}', '%Y/%m/%d %H:%M')
        messages.append(cls.makeMessafeFromLine(splitted, time))
      else:
        messages[-1].addMessage(line.replace('"', '').replace('\n', ''))
      
    return messages

  @classmethod
  def makeMessafeFromLine(cls, lineSplitted: list[str], time: str) -> Message:
    text = "".join(lineSplitted[2:]).replace('"','').replace('\n', '')
    if text.startswith("[") and text.endswith("]"):
      return Message(time, lineSplitted[1], text, Category.fromLabel(text))
    else:
      if text.startswith("[Poll]"):
        return Message(time, lineSplitted[1], text, Category.POLL)
      if text.startswith("☎ 通話時間") or text.startswith("☎ Call time"):
        return Message(time, lineSplitted[1], text, Category.CALL)
      elif text.startswith("☎ 通話をキャンセルしました") or text.startswith("☎ Canceled call"):
        return Message(time, lineSplitted[1], text, Category.CALL_CANCELLED)
      elif text.startswith("☎ 不在着信") or text.startswith("☎ No answer"):
        return Message(time, lineSplitted[1], text, Category.CALL_MISSED)
      elif lineSplitted[1] == "" and (text.endswith("unsent a message.") or text.endswith("メッセージの送信を取り消しました")):
        return Message(time, lineSplitted[1], text, Category.UNSENT)
      else:
        return Message(time, lineSplitted[1], text)



# Author: Yuji Sasaki <yuji@sasaki.dev>
# Copyright (c) 2022 Yuji Sasaki
# License: MIT License

from setuptools import setup
import os
import linelog2py

DESCRIPTION = "linelog2py: import LINE Talk History"
NAME = 'linelog2py'
AUTHOR = 'Yuji Sasaki'
AUTHOR_EMAIL = 'yuji@sasaki.dev'
URL = 'https://github.com/jyu0414/linelog2py'
LICENSE = 'MIT License'
DOWNLOAD_URL = 'https://github.com/jyu0414/linelog2py'
VERSION = linelog2py.__version__
PYTHON_REQUIRES = ">=3.6"

INSTALL_REQUIRES = [
]

EXTRAS_REQUIRE = {
}

PACKAGES = [
    'linelog2py'
]

CLASSIFIERS = [
    'Development Status :: 2 - Pre-Alpha',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3 :: Only',
    'Topic :: Scientific/Engineering :: Information Analysis',
    'Topic :: Text Editors :: Text Processing',
    'Topic :: Text Editors :: Word Processors'
]

with open(f'{os.environ["GITHUB_WORKSPACE"]}/README.md', 'r') as fp:
    readme = fp.read()
with open(f'{os.environ["GITHUB_WORKSPACE"]}/CONTACT.txt', 'r') as fp:
    contacts = fp.read()
long_description = readme + '\n\n' + contacts

setup(name=NAME,
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      maintainer=AUTHOR,
      maintainer_email=AUTHOR_EMAIL,
      description=DESCRIPTION,
      long_description=long_description,
      long_description_content_type='text/markdown',
      license=LICENSE,
      url=URL,
      version=VERSION,
      download_url=DOWNLOAD_URL,
      python_requires=PYTHON_REQUIRES,
      install_requires=INSTALL_REQUIRES,
      extras_require=EXTRAS_REQUIRE,
      packages=PACKAGES,
      classifiers=CLASSIFIERS
    )